@@ -142,8 +142,7 @@ members_changed_cb (EmpathyIndividualManager *manager,
 
   for (l = removed; l != NULL; l = g_list_next (l))
     {
-      if (individual_in_top_group_members (self, l->data) &&
-          !individual_should_be_in_top_group_members (self, l->data))
+      if (individual_in_top_group_members (self, l->data))
         self->priv->top_group_members = g_list_remove (
             self->priv->top_group_members, l->data);
 
@@ -284,7 +283,7 @@ empathy_roster_model_manager_constructed (GObject *object)
       G_CALLBACK (groups_changed_cb), self, 0);
   tp_g_signal_connect_object (self->priv->manager, "notify::top-individuals",
       G_CALLBACK (top_individuals_changed_cb), self, 0);
-  tp_g_signal_connect_object (self->priv->manager, "notify::favourites-changed",
+  tp_g_signal_connect_object (self->priv->manager, "favourites-changed",
       G_CALLBACK (favourites_changed_cb), self, 0);
 }
 
