@@ -28,8 +28,6 @@ enum
   SIG_INDIVIDUAL_ADDED,
   SIG_INDIVIDUAL_REMOVED,
   SIG_GROUPS_CHANGED,
-  SIG_TOP_INDIVIDUALS_CHANGED,
-  SIG_FAVOURITES_CHANGED,
   LAST_SIGNAL
 };
 
@@ -63,22 +61,6 @@ empathy_roster_model_default_init (EmpathyRosterModelInterface *iface)
         FOLKS_TYPE_INDIVIDUAL,
         G_TYPE_STRING,
         G_TYPE_BOOLEAN);
-
-  signals[SIG_TOP_INDIVIDUALS_CHANGED] =
-    g_signal_new ("top-individuals-changed",
-        EMPATHY_TYPE_ROSTER_MODEL,
-        G_SIGNAL_RUN_LAST,
-        0, NULL, NULL, NULL,
-        G_TYPE_NONE, 0);
-
-  signals[SIG_FAVOURITES_CHANGED] =
-    g_signal_new ("favourites-changed",
-        EMPATHY_TYPE_ROSTER_MODEL,
-        G_SIGNAL_RUN_LAST,
-        0, NULL, NULL, NULL,
-        G_TYPE_NONE, 2,
-        FOLKS_TYPE_INDIVIDUAL,
-        G_TYPE_BOOLEAN);
 }
 
 /***** Restricted *****/
@@ -107,22 +89,6 @@ empathy_roster_model_fire_groups_changed (EmpathyRosterModel *self,
       is_member);
 }
 
-void
-empathy_roster_model_fire_top_individuals_changed (EmpathyRosterModel *self)
-{
-  g_signal_emit (self, signals[SIG_TOP_INDIVIDUALS_CHANGED], 0);
-}
-
-void
-empathy_roster_model_fire_favourites_changed (EmpathyRosterModel *self,
-    FolksIndividual *individual,
-    gboolean favourite)
-{
-  g_signal_emit (self, signals[SIG_FAVOURITES_CHANGED], 0, individual,
-      favourite);
-}
-
-
 /***** Public *****/
 
 /**
@@ -169,49 +135,3 @@ empathy_roster_model_get_groups_for_individual (EmpathyRosterModel *self,
 
   return (* iface->get_groups_for_individual) (self, individual);
 }
-
-/**
- * empathy_roster_model_get_top_individuals:
- * @self: a #EmpathyRosterModel
- *
- * Returns a list of the top_individuals.
- *
- * Returns: (transfer none): a #GList of #FolksIndividual
- */
-GList *
-empathy_roster_model_get_top_individuals (EmpathyRosterModel *self)
-{
-  EmpathyRosterModelInterface *iface;
-
-  g_return_val_if_fail (EMPATHY_IS_ROSTER_MODEL (self), NULL);
-
-  iface = EMPATHY_ROSTER_MODEL_GET_IFACE (self);
-  g_return_val_if_fail (iface->get_top_individuals != NULL, NULL);
-
-  return (* iface->get_top_individuals) (self);
-}
-
-/**
- * empathy_roster_model_contact_in_top:
- * @self: a #EmpathyRosterModel
- * @contact: a #EmpathyRosterContact
- *
- * Checks if the passed #EmpathyRosterContact should be displayed in
- * top contacts.
- *
- * Returns: %TRUE if it should be displayed in top contacts, %FALSE
- * if not
- */
-gboolean
-empathy_roster_model_contact_in_top (EmpathyRosterModel *self,
-    EmpathyRosterContact *contact)
-{
-  EmpathyRosterModelInterface *iface;
-
-  g_return_val_if_fail (EMPATHY_IS_ROSTER_MODEL (self), FALSE);
-
-  iface = EMPATHY_ROSTER_MODEL_GET_IFACE (self);
-  g_return_val_if_fail (iface->contact_in_top != NULL, FALSE);
-
-  return (* iface->contact_in_top) (self, contact);
-}
